#define RK 1
#define __RK__ 1

#include <hourglass.h>
#include <rk.h>
#include <signal.h>

#define MAX_SETS 100

rk_resource_set_t *rs = NULL;

void make_res (unsigned int amount, 
	       unsigned int period,
	       int which)
{
  char name[RK_NAME_LEN];
  int result;

  if (!rs) {
    rs = (rk_resource_set_t *) malloc (MAX_SETS * sizeof (rk_resource_set_t));
    if (!rs) {
      printf ("oops: couldn't allocate resource set array\n");
      exit (-1);
    }
    memset (rs, 0, MAX_SETS * sizeof (rk_resource_set_t));
  }

  if (!rs[which]) {
    snprintf (name, RK_NAME_LEN, "hourglass_%d_thrd_%d", hourglass_pid, which);
    rs[which] = rk_resource_set_create(name);
    if (!rs[which]) {
      printf ("oops: could not create resource set\n");
      exit (-1);
    }
  }

  {
    struct cpu_reserve_attr attr;
    rk_reserve_t res;
    struct timespec c, t, b;

    c.tv_sec = amount/1000000, c.tv_nsec = (amount%1000000)*1000;
    t.tv_sec = period/1000000, t.tv_nsec = (period%1000000)*1000;
    b.tv_sec = 0, b.tv_nsec = 0;

    printf ("thread %d res: amount sec = %ld, nsec = %ld\n", 
	    which, c.tv_sec, c.tv_nsec);
    printf ("thread %d res: period sec = %ld, nsec = %ld\n", 
	    which, t.tv_sec, t.tv_nsec);

    attr.compute_time = c;
    attr.period = t;
    attr.deadline = t;
    attr.blocking_time = b;
    attr.start_time = b;
    attr.reserve_type.sch_mode = RSV_HARD;
    attr.reserve_type.enf_mode = RSV_HARD;
    attr.reserve_type.rep_mode = RSV_HARD;
    attr.processor = RK_ANY_CPU;

    result = rk_cpu_reserve_create(rs[which], &res, &attr);
  }

  if (result != 0) {
    printf ("oops: could not create CPU reservation\n");
    exit (-1);
  }

  /* attach the current process to the resource set */
  rk_resource_set_attach_process(rs[which], getpid());  
}

void delete_res (int which)
{
  if (rs[which]) {
    rk_resource_set_destroy (rs[which]);
  }
}
